#--------------------------------------------------------
# Shell名称:getctrl 
# 参数:无
# 返回值:
#  0:设备未被占用或无需要销毁的线程
#  1:正常销毁占用的线程 
#  -1:处理异常
# 说明:
#  检测打印机设备如果被强行占用，销毁除smartscanner以外所有的占用进程
# 做成:2021.09.16 ZYXX@MXD    Ver1.01
# 做成:2021.09.17 ZYXX@MXD    Ver1.02	
# 修改:2021.10.30 ZYXX@MXD    Ver1.03	Add VID&PID	
#--------------------------------------------------------
#! /bin/bash
#执行lsusb命令，获取标的打印机设备那一行的信息
lsusbretline=$(lsusb |grep -E "2a5c:6210|2a5c:6200")
if [ "${lsusbretline}" ];
then    
    #有标的的打印机设备
    echo "[getctrl]The info of printer device:${lsusbretline}"
    #获取设备的BusID和DeviceID
    businfo=${lsusbretline:4:3}
    deviceinfo=${lsusbretline:15:3}
    echo "[getctrl]BusID of printer:${businfo}"
    echo "[getctrl]DeviceID of printer:${deviceinfo}"

    #执行fuser命令,查看设备是否被占用
    fuserret=$(fuser -v /dev/bus/usb/"$businfo"/"$deviceinfo")
    echo "[getctrl]Command result of 'fuser':${fuserret}"     
    if [ -z "${fuserret}" ];
    then
        #标的打印机设备未被占用
        echo "[getctrl]The printer device is not occupied."
        #正常退出
        exit 0
    fi    
    
    #获取smartscanner的进程ID
    pidofsmartscanner=$(pidof -x "smartscanner")
    echo "[getctrl]The PID of 'smartscanner' process:${pidofsmartscanner}"      
    if [ -z "${pidofsmartscanner}" ];
    then
        #smartscanner进程不存在,直接销毁占用的所有其他进程
        killret=$(kill -9 ${fuserret})
        echo "[getctrl]1.Killed:${fuserret}"
        exit 1;
    fi
    
    #smartscanner进程存在的情况下，将其他占用的进程都销毁
    #查看占用进程中有无“smartscanner”
    #去除换行符等
    purefuserret="${fuserret//[$'\t\r\n']}"
    echo "[getctrl]***fuserret:${fuserret}"
    echo "[getctrl]***purefuserret:${purefuserret}"
    postion=$(expr index "${purefuserret}" "${pidofsmartscanner}")
    echo "[getctrl]Location of 'smartscanner' PID:${postion}"
    if [ "${postion}" -gt 1 ];
    then
        #占用进程中有“smartscanner”,把“smartscanner”的pid从字符串中删除
        purepid=${purefuserret//${pidofsmartscanner}/' '}
        echo "[getctrl]Character information after removing 'smartscanner' PID:${purepid}"
        if [ ${#purepid} -lt 4 ]
        then
            #截取后的字符串长度小于4,无有效PID，直接退出
            echo "[getctrl]There is no PID to kill."
            exit 0
        fi
        #销毁除smartscanner以外所有的进程
        killret=$(kill -9 ${purepid})
        echo "[getctrl]2.Killed:${purepid}"
        exit 1;
    else
        #占用进程中无“smartscanner”,直接销毁所有进程
        if [ ${purefuserret} -lt 4 ];
        then
            #字符串长度小于4,无有效PID，直接退出
            echo "[getctrl]There is no PID to kill."
            exit 0
        fi
        #销毁除smartscanner以外所有的进程
        killret=$(kill -9 ${purefuserret})
        echo "[getctrl]2.Killed:${fuserret}"
        exit 1;        
    fi    
else
    #标的打印机设备未启动
    echo "[getctrl]The printer device is not started." 
    #正常退出
    exit 0
fi
